@@ -1,9 +1,16 @@
 # Apache::ConfigParser::Directive: A single Apache directive or start context.
 #
-# Copyright (C) 2001-2002 Blair Zajac.  All rights reserved.
+# $HeadURL: http://www.orcaware.com/svn/repos/tags/perl_apache_configparser/1.00/lib/Apache/ConfigParser/Directive.pm $
+# $LastChangedRevision: 435 $
+# $LastChangedDate: 2005-03-26 21:48:58 -0800 (Sat, 26 Mar 2005) $
+# $LastChangedBy: blair@orcaware.com $
+#
+# Copyright (C) 2001-2005 Blair Zajac.  All rights reserved.
 
 package Apache::ConfigParser::Directive;
+
 require 5.004_05;
+
 use strict;
 use Exporter;
 use Carp;
@@ -12,7 +19,7 @@ use Tree::DAG_Node 1.04;
 
 use vars qw(@EXPORT_OK @ISA $VERSION);
 @ISA     = qw(Tree::DAG_Node Exporter);
-$VERSION = sprintf '%d.%02d', '$Revision: 0.03 $' =~ /(\d+)\.(\d+)/;
+$VERSION = '1.00';
 
 # Determine if the filenames are case sensitive.
 use constant CASE_SENSITIVE_PATH => (! File::Spec->case_tolerant);
@@ -65,11 +72,11 @@ push(@EXPORT_OK, qw(%directive_value_takes_abs_path
   $d->value('"%h %l %u %t \"%r\" %>s %b" common');
 
   # Get a string form of the name.
-  # Prints `logformat'.
+  # Prints 'logformat'.
   print $d->name, "\n";
 
   # Get a string form of the value.
-  # Prints `"%h %l %u %t \"%r\" %>s %b" common'.
+  # Prints '"%h %l %u %t \"%r\" %>s %b" common'.
   print $d->value, "\n";
 
   # Get the values separated into individual elements.  Whitespace
@@ -90,7 +97,7 @@ push(@EXPORT_OK, qw(%directive_value_takes_abs_path
   $d->value('"%{Referer}i -> %U" referer');
 
   # There are also an equivalent pair of values that are called
-  # `original' that can be accessed via orig_value,
+  # 'original' that can be accessed via orig_value,
   # get_orig_value_array and set_orig_value_array.
   $d->orig_value('"%{User-agent}i" agent');
   $d->set_orig_value_array('%{User-agent}i', 'agent');
@@ -117,15 +124,15 @@ This module holds a directive or context:
   name
   value in string form
   value in array form
-  a separate value termed `original' in string form
-  a separate value termed `original' in array form
+  a separate value termed 'original' in string form
+  a separate value termed 'original' in array form
   the filename where the directive was set
   the line number in the filename where the directive was set
 
-The `original' value is separate from the non-`original' value and the
+The 'original' value is separate from the non-'original' value and the
 methods to operate on the two sets of values have distinct names.  The
-`original' value can be used to store the original value of a
-directive while the non-`directive' value can be a modified form, such
+'original' value can be used to store the original value of a
+directive while the non-'directive' value can be a modified form, such
 as changing the CustomLog filename to make it absolute.  The actual
 use of these two distinct values is up to the caller as this module
 does not link the two in any way.
@@ -180,7 +187,8 @@ version of I<$name> and return the original name.
 
 sub name {
   unless (@_ < 3) {
-    confess "$0: Apache::ConfigParser::Directive::name $INCORRECT_NUMBER_OF_ARGS";
+    confess "$0: Apache::ConfigParser::Directive::name ",
+            $INCORRECT_NUMBER_OF_ARGS;
   }
 
   my $self = shift;
@@ -215,16 +223,17 @@ honored to not begin or end a value element.
 =item $d->orig_value($value)
 
 Identical behavior as C<value>, except that this applies to a the
-`original' value.  Use C<orig_value_ref> or C<get_orig_value_array> to
+'original' value.  Use C<orig_value_ref> or C<get_orig_value_array> to
 get the value elements.
 
 =cut
 
 # This function manages getting and setting the string value for
-# either the `value' or `orig_value' hash keys.
+# either the 'value' or 'orig_value' hash keys.
 sub _get_set_value_string {
   unless (@_ > 1 and @_ < 4) {
-    confess "$0: Apache::ConfigParser::Directive::_get_set_value_string $INCORRECT_NUMBER_OF_ARGS";
+    confess "$0: Apache::ConfigParser::Directive::_get_set_value_string ",
+            $INCORRECT_NUMBER_OF_ARGS;
   }
 
   my $self            = shift;
@@ -277,7 +286,8 @@ sub _get_set_value_string {
 
 sub value {
   unless (@_ and @_ < 3) {
-    confess "$0: Apache::ConfigParser::Directive::value $INCORRECT_NUMBER_OF_ARGS";
+    confess "$0: Apache::ConfigParser::Directive::value ",
+            $INCORRECT_NUMBER_OF_ARGS;
   }
 
   return _get_set_value_string(@_, 'value');
@@ -285,7 +295,8 @@ sub value {
 
 sub orig_value {
   unless (@_ and @_ < 3) {
-    confess "$0: Apache::ConfigParser::Directive::orig_value $INCORRECT_NUMBER_OF_ARGS";
+    confess "$0: Apache::ConfigParser::Directive::orig_value ",
+            $INCORRECT_NUMBER_OF_ARGS;
   }
 
   return _get_set_value_string(@_, 'orig_value');
@@ -310,7 +321,7 @@ string returned from C<value> will not be consistent with the array.
 =item $d->orig_value_array_ref(\@array)
 
 Identical behavior as C<value_array_ref>, except that this applies to
-the `original' value.
+the 'original' value.
 
 =cut
 
@@ -319,7 +330,8 @@ the `original' value.
 # suitable for placing into an Apache configuration file.
 sub _set_value_array {
   unless (@_ > 1) {
-    confess "$0: Apache::ConfigParser::Directive::_set_value_array $INCORRECT_NUMBER_OF_ARGS";
+    confess "$0: Apache::ConfigParser::Directive::_set_value_array ",
+            $INCORRECT_NUMBER_OF_ARGS;
   }
 
   my $self            = shift;
@@ -354,7 +366,8 @@ sub _set_value_array {
 
 sub value_array_ref {
   unless (@_ and @_ < 3) {
-    confess "$0: Apache::ConfigParser::Directive::value_array_ref $INCORRECT_NUMBER_OF_ARGS";
+    confess "$0: Apache::ConfigParser::Directive::value_array_ref ",
+            $INCORRECT_NUMBER_OF_ARGS;
   }
 
   my $self = shift;
@@ -376,7 +389,8 @@ sub value_array_ref {
 
 sub orig_value_array_ref {
   unless (@_ and @_ < 3) {
-    confess "$0: Apache::ConfigParser::Directive::orig_value_array_ref $INCORRECT_NUMBER_OF_ARGS";
+    confess "$0: Apache::ConfigParser::Directive::orig_value_array_ref ",
+            $INCORRECT_NUMBER_OF_ARGS;
   }
 
   my $self = shift;
@@ -406,13 +420,14 @@ nothing in a void context.
 =item $d->get_orig_value_array
 
 This has the same behavior of C<get_value_array> except that it
-operates on the `original' value.
+operates on the 'original' value.
 
 =cut
 
 sub get_value_array {
   unless (@_ == 1) {
-    confess "$0: Apache::ConfigParser::Directive::get_value_array $INCORRECT_NUMBER_OF_ARGS";
+    confess "$0: Apache::ConfigParser::Directive::get_value_array ",
+            $INCORRECT_NUMBER_OF_ARGS;
   }
 
   my $ref = shift->{value_array};
@@ -426,7 +441,8 @@ sub get_value_array {
 
 sub get_orig_value_array {
   unless (@_ == 1) {
-    confess "$0: Apache::ConfigParser::Directive::get_orig_value_array $INCORRECT_NUMBER_OF_ARGS";
+    confess "$0: Apache::ConfigParser::Directive::get_orig_value_array ",
+            $INCORRECT_NUMBER_OF_ARGS;
   }
 
   my $ref = shift->{orig_value_array};
@@ -457,7 +473,7 @@ the value string.
 =item $d->set_orig_value_array(@values)
 
 This has the same behavior as C<set_value_array> except that it
-operates on the `original' value.
+operates on the 'original' value.
 
 =cut
 
@@ -498,13 +514,13 @@ These are the differences between the methods:
 
 =over 4
 
-1) The methods beginning with the string `value_is' apply to the
+1) The methods beginning with the string 'value_is' apply to the
 current value in the directive while the methods beginning with the
-string `orig_value_is' apply to the original value of the directive.
+string 'orig_value_is' apply to the original value of the directive.
 
-2) The methods `*value_is_path' test if the directive value is a path,
-either absolute or relative.  The methods `*value_is_abs_path' test if
-the path if an absolute path, and the methods `*value_is_rel_path'
+2) The methods '*value_is_path' test if the directive value is a path,
+either absolute or relative.  The methods '*value_is_abs_path' test if
+the path if an absolute path, and the methods '*value_is_rel_path'
 test if the path is not an absolute path.
 
 =back
@@ -542,7 +558,9 @@ sub CHECK_TYPE_ABS_OR_REL () { 'abs_or_rel' }
 # value_is_rel_path and orig_value_is_rel_path.
 sub _value_is_path_or_abs_path_or_rel_path {
   unless (@_ == 4) {
-    confess "$0: Apache::ConfigParser::Directive::_value_is_path_or_abs_path_or_rel_path $INCORRECT_NUMBER_OF_ARGS";
+    confess "$0: Apache::ConfigParser::Directive::",
+            "_value_is_path_or_abs_path_or_rel_path ",
+            $INCORRECT_NUMBER_OF_ARGS;
   }
 
   my ($self,
@@ -553,11 +571,15 @@ sub _value_is_path_or_abs_path_or_rel_path {
   unless ($check_type eq CHECK_TYPE_ABS or
           $check_type eq CHECK_TYPE_REL or
           $check_type eq CHECK_TYPE_ABS_OR_REL) {
-    confess "$0: Apache::ConfigParser::Directive::_value_is_path_or_abs_path_or_rel_path passed invalid check_type value `$check_type'.\n";
+    confess "$0: Apache::ConfigParser::Directive::",
+            "_value_is_path_or_abs_path_or_rel_path ",
+            "passed invalid check_type value '$check_type'.\n";
   }
 
   if (defined $value_path_index and $value_path_index !~ /^\d+$/) {
-    confess "$0: Apache::ConfigParser::Directive::_value_is_path_or_abs_path_or_rel_path passed invalid value_path_index value `$value_path_index'.\n";
+    confess "$0: Apache::ConfigParser::Directive::",
+            "_value_is_path_or_abs_path_or_rel_path ",
+            "passed invalid value_path_index value '$value_path_index'.\n";
   }
 
   my $array_ref = $self->{$array_var_name};
@@ -616,7 +638,7 @@ sub _value_is_path_or_abs_path_or_rel_path {
       $sub_ref = $directive_value_takes_rel_path{$directive_name};
     }
   } else {
-    confess "$0: internal error: check_type case `$check_type' not handled.\n";
+    confess "$0: internal error: check_type case '$check_type' not handled.\n";
   }
 
   unless ($sub_ref) {
@@ -632,7 +654,8 @@ sub _value_is_path_or_abs_path_or_rel_path {
     } elsif ($check_type eq CHECK_TYPE_REL) {
       return File::Spec->file_name_is_absolute($path) ? 0 : 1;
     } else {
-      confess "$0: internal error: check_type case `$check_type' not handled.\n";
+      confess "$0: internal error: check_type case ",
+              "'$check_type' not handled.\n";
     }
   } else {
     return 0;
@@ -641,7 +664,8 @@ sub _value_is_path_or_abs_path_or_rel_path {
 
 sub value_is_path {
   unless (@_ < 3) {
-    confess "$0: Apache::ConfigParser::Directive::value_is_path $INCORRECT_NUMBER_OF_ARGS";
+    confess "$0: Apache::ConfigParser::Directive::value_is_path ",
+            $INCORRECT_NUMBER_OF_ARGS;
   }
 
   _value_is_path_or_abs_path_or_rel_path($_[0],
@@ -655,13 +679,14 @@ sub value_is_path {
 =item $d->orig_value_is_path($index_into_value_array)
 
 This has the same behavior as C<$d->value_is_path> except the results
-are applicable to C<$d>'s `original' value array.
+are applicable to C<$d>'s 'original' value array.
 
 =cut
 
 sub orig_value_is_path {
   unless (@_ < 3) {
-    confess "$0: Apache::ConfigParser::Directive::orig_value_is_path $INCORRECT_NUMBER_OF_ARGS";
+    confess "$0: Apache::ConfigParser::Directive::orig_value_is_path ",
+            $INCORRECT_NUMBER_OF_ARGS;
   }
 
   _value_is_path_or_abs_path_or_rel_path($_[0],
@@ -694,7 +719,8 @@ have different path formats, such as Unix, Windows and Macintosh.
 
 sub value_is_abs_path {
   unless (@_ < 3) {
-    confess "$0: Apache::ConfigParser::Directive::value_is_abs_path $INCORRECT_NUMBER_OF_ARGS";
+    confess "$0: Apache::ConfigParser::Directive::value_is_abs_path ",
+            $INCORRECT_NUMBER_OF_ARGS;
   }
 
   _value_is_path_or_abs_path_or_rel_path($_[0],
@@ -708,13 +734,14 @@ sub value_is_abs_path {
 =item $d->orig_value_is_abs_path($index_into_value_array)
 
 This has the same behavior as C<$d->value_is_abs_path> except the
-results are applicable to C<$d>'s `original' value array.
+results are applicable to C<$d>'s 'original' value array.
 
 =cut
 
 sub orig_value_is_abs_path {
   unless (@_ < 3) {
-    confess "$0: Apache::ConfigParser::Directive::orig_value_is_abs_path $INCORRECT_NUMBER_OF_ARGS";
+    confess "$0: Apache::ConfigParser::Directive::orig_value_is_abs_path ",
+            $INCORRECT_NUMBER_OF_ARGS;
   }
 
   _value_is_path_or_abs_path_or_rel_path($_[0],
@@ -747,7 +774,8 @@ have different path formats, such as Unix, Windows and Macintosh.
 
 sub value_is_rel_path {
   unless (@_ < 3) {
-    confess "$0: Apache::ConfigParser::Directive::value_is_rel_path $INCORRECT_NUMBER_OF_ARGS";
+    confess "$0: Apache::ConfigParser::Directive::value_is_rel_path ",
+            $INCORRECT_NUMBER_OF_ARGS;
   }
 
   _value_is_path_or_abs_path_or_rel_path($_[0],
@@ -761,13 +789,14 @@ sub value_is_rel_path {
 =item $d->orig_value_is_rel_path($index_into_value_array)
 
 This has the same behavior as C<$d->value_is_rel_path> except the
-results are applicable to C<$d>'s `original' value array.
+results are applicable to C<$d>'s 'original' value array.
 
 =cut
 
 sub orig_value_is_rel_path {
   unless (@_ < 3) {
-    confess "$0: Apache::ConfigParser::Directive::orig_value_is_rel_path $INCORRECT_NUMBER_OF_ARGS";
+    confess "$0: Apache::ConfigParser::Directive::orig_value_is_rel_path ",
+            $INCORRECT_NUMBER_OF_ARGS;
   }
 
   _value_is_path_or_abs_path_or_rel_path($_[0],
@@ -788,7 +817,8 @@ directive or context and return the original filename.
 
 sub filename {
   unless (@_ < 3) {
-    confess "$0: Apache::ConfigParser::Directive::filename $INCORRECT_NUMBER_OF_ARGS";
+    confess "$0: Apache::ConfigParser::Directive::filename ",
+            $INCORRECT_NUMBER_OF_ARGS;
   }
 
   my $self = shift;
@@ -813,7 +843,8 @@ the directive or context and return the original line number.
 
 sub line_number {
   unless (@_ < 3) {
-    confess "$0: Apache::ConfigParser::Directive::line_number $INCORRECT_NUMBER_OF_ARGS";
+    confess "$0: Apache::ConfigParser::Directive::line_number ",
+            $INCORRECT_NUMBER_OF_ARGS;
   }
 
   my $self = shift;
@@ -1177,7 +1208,7 @@ Blair Zajac <blair@orcaware.com>.
 
 =head1 COPYRIGHT
 
-Copyright (C) 2001-2002 Blair Zajac.  All rights reserved.  This
+Copyright (C) 2001-2005 Blair Zajac.  All rights reserved.  This
 program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
 
